@@ -18,8 +18,40 @@ Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301 USA
 """
 
+# pylint: disable=import-outside-toplevel
+
 
 def guake_version():
-    # Do not import in the module root to speed up the dbus communication as much as possible
-    import pbr.version
-    return pbr.version.VersionInfo('guake').version_string()
+    from ._version import version
+
+    return version
+
+
+def vte_version():
+    import gi
+
+    gi.require_version("Vte", "2.91")
+
+    from gi.repository import Vte
+
+    s = f"{Vte.MAJOR_VERSION}.{Vte.MINOR_VERSION}.{Vte.MICRO_VERSION}"
+    return s
+
+
+def vte_runtime_version():
+    import gi
+
+    gi.require_version("Vte", "2.91")
+
+    from gi.repository import Vte
+
+    return f"{Vte.get_major_version()}.{Vte.get_minor_version()}.{Vte.get_micro_version()}"
+
+
+def gtk_version():
+    import gi
+
+    gi.require_version("Gtk", "3.0")
+    from gi.repository import Gtk
+
+    return f"{Gtk.MAJOR_VERSION}.{Gtk.MINOR_VERSION}.{Gtk.MICRO_VERSION}"
