@@ -1,12 +1,10 @@
 import logging
 import time
 
-from locale import gettext as _
-
 import gi
-gi.require_version('Vte', '2.91')  # vte-0.42
-gi.require_version('Gtk', '3.0')
-from gi.repository import GLib
+
+gi.require_version("Vte", "2.91")  # vte-0.42
+gi.require_version("Gtk", "3.0")
 from gi.repository import GObject
 from gi.repository import Gdk
 from gi.repository import Gio
@@ -15,7 +13,9 @@ from gi.repository import Vte
 
 from guake.callbacks import MenuHideCallback
 from guake.callbacks import TerminalContextMenuCallbacks
+from guake.dialogs import PromptResetColorsDialog
 from guake.dialogs import RenameDialog
+from guake.globals import PCRE2_MULTILINE
 from guake.menus import mk_tab_context_menu
 from guake.menus import mk_terminal_context_menu
 from guake.utils import HidePrevention
@@ -28,42 +28,41 @@ log = logging.getLogger(__name__)
 # TODO remove calls to guake
 
 
-class TerminalHolder():
+class TerminalHolder:
     UP = 0
     DOWN = 1
     RIGHT = 2
     LEFT = 3
 
     def get_terminals(self):
-        pass
+        raise NotImplementedError
 
     def iter_terminals(self):
-        pass
+        raise NotImplementedError
 
     def replace_child(self, old, new):
-        pass
+        raise NotImplementedError
 
     def get_guake(self):
-        pass
+        raise NotImplementedError
 
     def get_window(self):
-        pass
+        raise NotImplementedError
 
     def get_settings(self):
-        pass
+        raise NotImplementedError
 
     def get_root_box(self):
-        pass
+        raise NotImplementedError
 
     def get_notebook(self):
-        pass
+        raise NotImplementedError
 
     def remove_dead_child(self, child):
-        pass
+        raise NotImplementedError
 
 
 class RootTerminalBox(Gtk.Overlay, TerminalHolder):
-
     def __init__(self, guake, parent_notebook):
         super().__init__()
         self.guake = guake
@@ -76,36 +75,36 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
         self._add_search_box()
 
     def _add_search_box(self):
-        """ --------------------------------------|
-            | Revealer                            |
-            | |-----------------------------------|
-            | | Frame                             |
-            | | |---------------------------------|
-            | | | HBox                            |
-            | | | |---| |-------| |----| |------| |
-            | | | | x | | Entry | |Prev| | Next | |
-            | | | |---| |-------| |----| |------| |
-            --------------------------------------|
+        """--------------------------------------|
+        | Revealer                            |
+        | |-----------------------------------|
+        | | Frame                             |
+        | | |---------------------------------|
+        | | | HBox                            |
+        | | | |---| |-------| |----| |------| |
+        | | | | x | | Entry | |Prev| | Next | |
+        | | | |---| |-------| |----| |------| |
+        --------------------------------------|
         """
         self.search_revealer = Gtk.Revealer()
-        self.search_frame = Gtk.Frame(name='search-frame')
+        self.search_frame = Gtk.Frame(name="search-frame")
         self.search_box = Gtk.HBox()
 
         # Search
         self.search_close_btn = Gtk.Button()
         self.search_close_btn.set_can_focus(False)
-        close_icon = Gio.ThemedIcon(name='window-close-symbolic')
+        close_icon = Gio.ThemedIcon(name="window-close-symbolic")
         close_image = Gtk.Image.new_from_gicon(close_icon, Gtk.IconSize.BUTTON)
         self.search_close_btn.set_image(close_image)
         self.search_entry = Gtk.SearchEntry()
         self.search_prev_btn = Gtk.Button()
         self.search_prev_btn.set_can_focus(False)
-        prev_icon = Gio.ThemedIcon(name='go-up-symbolic')
+        prev_icon = Gio.ThemedIcon(name="go-up-symbolic")
         prev_image = Gtk.Image.new_from_gicon(prev_icon, Gtk.IconSize.BUTTON)
         self.search_prev_btn.set_image(prev_image)
         self.search_next_btn = Gtk.Button()
         self.search_next_btn.set_can_focus(False)
-        next_icon = Gio.ThemedIcon(name='go-down-symbolic')
+        next_icon = Gio.ThemedIcon(name="go-down-symbolic")
         next_image = Gtk.Image.new_from_gicon(next_icon, Gtk.IconSize.BUTTON)
         self.search_next_btn.set_image(next_image)
 
@@ -120,16 +119,15 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
 
         # Frame
         self.search_frame.set_margin_end(12)
-        self.search_frame.get_style_context().add_class('background')
+        self.search_frame.get_style_context().add_class("background")
         css_provider = Gtk.CssProvider()
         css_provider.load_from_data(
-            b'#search-frame border {'
-            b'    padding: 5px 5px 5px 5px;'
-            b'    border: none;'
-            b'}'
+            b"#search-frame border {" b"    padding: 5px 5px 5px 5px;" b"    border: none;" b"}"
         )
         Gtk.StyleContext.add_provider_for_screen(
-            Gdk.Screen.get_default(), css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
+            Gdk.Screen.get_default(),
+            css_provider,
+            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION,
         )
 
         # Add to revealer
@@ -143,16 +141,27 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
         self.add_overlay(self.search_revealer)
 
         # Events
-        self.search_entry.connect('key-press-event', self.on_search_entry_keypress)
-        self.search_entry.connect('changed', self.set_search)
-        self.search_entry.connect('activate', self.do_search)
-        self.search_entry.connect('focus-in-event', self.on_search_entry_focus_in)
-        self.search_entry.connect('focus-out-event', self.on_search_entry_focus_out)
-        self.search_next_btn.connect('clicked', self.on_search_next_clicked)
-        self.search_prev_btn.connect('clicked', self.on_search_prev_clicked)
-        self.search_close_btn.connect('clicked', self.close_search_box)
+        self.search_entry.connect("key-press-event", self.on_search_entry_keypress)
+        self.search_entry.connect("changed", self.set_search)
+        self.search_entry.connect("activate", self.do_search)
+        self.search_entry.connect("focus-in-event", self.on_search_entry_focus_in)
+        self.search_entry.connect("focus-out-event", self.on_search_entry_focus_out)
+        self.search_next_btn.connect("clicked", self.on_search_next_clicked)
+        self.search_prev_btn.connect("clicked", self.on_search_prev_clicked)
+        self.search_close_btn.connect("clicked", self.close_search_box)
         self.search_prev = True
 
+        # Search revealer visible
+        def search_revealer_show_cb(widget):
+            if not widget.get_child_revealed():
+                widget.hide()
+
+        self.search_revealer.hide()
+        self.search_revealer_show_cb_id = self.search_revealer.connect(
+            "show", search_revealer_show_cb
+        )
+        self.search_frame.connect("unmap", lambda x: self.search_revealer.hide())
+
     def get_terminals(self):
         return self.get_child().get_terminals()
 
@@ -166,18 +175,11 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
         self.set_child(new)
 
     def set_child(self, terminal_holder):
-        if isinstance(terminal_holder, TerminalHolder) or True:
+        if isinstance(terminal_holder, TerminalHolder):
             self.child = terminal_holder
             self.add(self.child)
         else:
-            print(
-                "wtf, what have you added to me???"
-                "(RootTerminalBox.add(%s))" % type(terminal_holder)
-            )
-
-    def focus():
-        if self.get_terminals():
-            self.get_terminals()[0].grab_focus()
+            raise RuntimeError(f"Error adding (RootTerminalBox.add({type(terminal_holder)}))")
 
     def get_child(self):
         return self.child
@@ -196,24 +198,29 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
 
     def save_box_layout(self, box, panes: list):
         """Save box layout with pre-order traversal, it should result `panes` with
-           a full binary tree in list.
+        a full binary tree in list.
         """
         if not box:
-            panes.append({'type': None, 'directory': None})
+            panes.append({"type": None, "directory": None})
             return
         if isinstance(box, DualTerminalBox):
-            btype = 'dual' + ('_h' if box.orient is DualTerminalBox.ORIENT_V else '_v')
-            panes.append({'type': btype, 'directory': None})
+            btype = "dual" + ("_h" if box.orient is DualTerminalBox.ORIENT_V else "_v")
+            panes.append({"type": btype, "directory": None})
             self.save_box_layout(box.get_child1(), panes)
             self.save_box_layout(box.get_child2(), panes)
         elif isinstance(box, TerminalBox):
-            btype = 'term'
+            btype = "term"
             directory = box.terminal.get_current_directory()
-            panes.append({'type': btype, 'directory': directory})
+            panes.append(
+                {
+                    "type": btype,
+                    "directory": directory,
+                    "custom_colors": box.terminal.get_custom_colors_dict(),
+                }
+            )
 
     def restore_box_layout(self, box, panes: list):
-        """Restore box layout by `panes`
-        """
+        """Restore box layout by `panes`"""
         if not panes or not isinstance(panes, list):
             return
         if not box or not isinstance(box, TerminalBox):
@@ -221,7 +228,7 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
             return
 
         cur = panes.pop(0)
-        if cur['type'].startswith('dual'):
+        if cur["type"].startswith("dual"):
             while True:
                 if self.guake:
                     # If Guake are not visible, we should pending the restore, then do the
@@ -230,8 +237,9 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
                     # Otherwise we will stuck in the infinite loop, since new DualTerminalBox
                     # cannot get any allocation when Guake is invisible
                     if (
-                        not self.guake.window.get_property('visible') or self.get_notebook() is
-                        not self.guake.notebook_manager.get_current_notebook()
+                        not self.guake.window.get_property("visible")
+                        or self.get_notebook()
+                        is not self.guake.notebook_manager.get_current_notebook()
                     ):
                         panes.insert(0, cur)
                         self.guake._failed_restore_page_split.append((self, box, panes))
@@ -240,7 +248,7 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
                 # UI didn't update, wait for it
                 alloc = box.get_allocation()
                 if alloc.width == 1 and alloc.height == 1:
-                    time.sleep(.01)
+                    time.sleep(0.01)
                 else:
                     break
 
@@ -248,7 +256,7 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
                 while Gtk.events_pending():
                     Gtk.main_iteration()
 
-            if cur['type'].endswith('v'):
+            if cur["type"].endswith("v"):
                 box = box.split_v()
             else:
                 box = box.split_h()
@@ -266,7 +274,8 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
                 box.unset_terminal()
 
             # Replace term in the TerminalBox
-            term = self.get_notebook().terminal_spawn(cur['directory'])
+            term = self.get_notebook().terminal_spawn(cur["directory"])
+            term.set_custom_colors_from_dict(cur.get("custom_colors", None))
             box.set_terminal(term)
             self.get_notebook().terminal_attached(term)
 
@@ -284,24 +293,22 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
         page_num = self.get_notebook().page_num(self)
         self.get_notebook().remove_page(page_num)
 
-    def move_focus(self, direction, fromChild):
-        pass
-
     def block_notebook_on_button_press_id(self):
         GObject.signal_handler_block(
-            self.get_notebook(),
-            self.get_notebook().notebook_on_button_press_id
+            self.get_notebook(), self.get_notebook().notebook_on_button_press_id
         )
 
     def unblock_notebook_on_button_press_id(self):
         GObject.signal_handler_unblock(
-            self.get_notebook(),
-            self.get_notebook().notebook_on_button_press_id
+            self.get_notebook(), self.get_notebook().notebook_on_button_press_id
         )
 
     def show_search_box(self):
         if not self.search_revealer.get_reveal_child():
+            GObject.signal_handler_block(self.search_revealer, self.search_revealer_show_cb_id)
+            self.search_revealer.set_visible(True)
             self.search_revealer.set_reveal_child(True)
+            GObject.signal_handler_unblock(self.search_revealer, self.search_revealer_show_cb_id)
             # XXX: Mestery line to avoid Gtk-CRITICAL stuff
             # (guake:22694): Gtk-CRITICAL **: 18:04:57.345:
             # gtk_widget_event: assertion 'WIDGET_REALIZED_FOR_EVENT (widget, event)' failed
@@ -337,9 +344,9 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
 
     def on_search_entry_keypress(self, widget, event):
         key = Gdk.keyval_name(event.keyval)
-        if key == 'Escape':
+        if key == "Escape":
             self.hide_search_box()
-        elif key == 'Return':
+        elif key == "Return":
             # Combine with Shift?
             if event.state & Gdk.ModifierType.SHIFT_MASK:
                 self.search_prev = False
@@ -348,7 +355,7 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
                 self.search_prev = True
 
     def reset_term_search(self, term):
-        term.search_set_gregex(GLib.Regex('', 0, 0), 0)
+        term.search_set_regex(None, 0)
         term.search_find_next()
 
     def set_search(self, widget):
@@ -363,9 +370,10 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
 
             # Set search regex on term
             self.searchstring = text
-            self.searchre = GLib.Regex(text, 0, 0)
-            term.search_set_gregex(self.searchre, 0)
-
+            self.searchre = Vte.Regex.new_for_search(
+                text, -1, Vte.REGEX_FLAGS_DEFAULT | PCRE2_MULTILINE
+            )
+            term.search_set_regex(self.searchre, 0)
         self.do_search(None)
 
     def do_search(self, widget):
@@ -377,16 +385,14 @@ class RootTerminalBox(Gtk.Overlay, TerminalHolder):
 
 class TerminalBox(Gtk.Box, TerminalHolder):
 
-    """A box to group the terminal and a scrollbar.
-    """
+    """A box to group the terminal and a scrollbar."""
 
     def __init__(self):
         super().__init__(orientation=Gtk.Orientation.HORIZONTAL)
         self.terminal = None
 
     def set_terminal(self, terminal):
-        """Packs the terminal widget.
-        """
+        """Packs the terminal widget."""
         if self.terminal is not None:
             raise RuntimeError("TerminalBox: terminal already set")
         self.terminal = terminal
@@ -397,22 +403,21 @@ class TerminalBox(Gtk.Box, TerminalHolder):
             self.terminal.connect("button-press-event", self.on_button_press, None)
         )
         self.terminal.handler_ids.append(
-            self.terminal.connect('child-exited', self.on_terminal_exited)
+            self.terminal.connect("child-exited", self.on_terminal_exited)
         )
         self.pack_start(self.terminal, True, True, 0)
         self.terminal.show()
         self.add_scroll_bar()
 
     def add_scroll_bar(self):
-        """Packs the scrollbar.
-        """
+        """Packs the scrollbar."""
         adj = self.terminal.get_vadjustment()
         self.scroll = Gtk.VScrollbar(adj)
         self.scroll.show()
         self.pack_start(self.scroll, False, False, 0)
 
         self.terminal.handler_ids.append(
-            self.terminal.connect('scroll-event', self.__scroll_event_cb)
+            self.terminal.connect("scroll-event", self.__scroll_event_cb)
         )
 
     def __scroll_event_cb(self, widget, event):
@@ -420,8 +425,8 @@ class TerminalBox(Gtk.Box, TerminalHolder):
         adj = self.scroll.get_adjustment()
         page_size = adj.get_page_size()
         if (
-            event.get_state() & Gdk.ModifierType.SHIFT_MASK and
-            event.get_state() & Gdk.ModifierType.CONTROL_MASK
+            event.get_state() & Gdk.ModifierType.SHIFT_MASK
+            and event.get_state() & Gdk.ModifierType.CONTROL_MASK
         ):
             # Ctrl + Shift + Mouse Scroll (4 pages)
             adj.set_page_increment(page_size * 40)
@@ -474,8 +479,8 @@ class TerminalBox(Gtk.Box, TerminalHolder):
         parent.replace_child(self, dual_terminal_box)
         dual_terminal_box.set_child_first(self)
         dual_terminal_box.set_child_second(terminal_box)
+        terminal_box.show()
         dual_terminal_box.show()
-        dual_terminal_box.show_all()
         notebook.terminal_attached(terminal)
 
         return dual_terminal_box
@@ -496,7 +501,7 @@ class TerminalBox(Gtk.Box, TerminalHolder):
         return self.get_parent().get_notebook()
 
     def remove_dead_child(self, child):
-        print("Can't do, have no \"child\"")
+        print('Can\'t do, have no "child"')
 
     def on_terminal_focus(self, *args):
         self.get_root_box().set_last_terminal_focused(self.terminal)
@@ -509,16 +514,22 @@ class TerminalBox(Gtk.Box, TerminalHolder):
     def on_button_press(self, target, event, user_data):
         if event.button == 3:
             # First send to background process if handled, do nothing else
-            if not event.get_state() & Gdk.ModifierType.SHIFT_MASK:
-                if Vte.Terminal.do_button_press_event(self.terminal, event):
-                    return True
+            if (
+                not event.get_state() & Gdk.ModifierType.SHIFT_MASK
+                and Vte.Terminal.do_button_press_event(self.terminal, event)
+            ):
+                return True
 
             menu = mk_terminal_context_menu(
-                self.terminal, self.get_window(), self.get_settings(),
+                self.terminal,
+                self.get_window(),
+                self.get_settings(),
                 TerminalContextMenuCallbacks(
-                    self.terminal, self.get_window(), self.get_settings(),
-                    self.get_root_box().get_notebook()
-                )
+                    self.terminal,
+                    self.get_window(),
+                    self.get_settings(),
+                    self.get_root_box().get_notebook(),
+                ),
             )
             menu.connect("hide", MenuHideCallback(self.get_window()).on_hide)
             HidePrevention(self.get_window()).prevent()
@@ -595,7 +606,10 @@ class DualTerminalBox(Gtk.Paned, TerminalHolder):
 
     def grab_box_terminal_focus(self, box):
         if isinstance(box, DualTerminalBox):
-            next(box.iter_terminals()).grab_focus()
+            try:
+                next(box.iter_terminals()).grab_focus()
+            except StopIteration:
+                log.error("Both panes are empty")
         else:
             box.get_terminal().grab_focus()
 
@@ -615,7 +629,6 @@ class DualTerminalBox(Gtk.Paned, TerminalHolder):
 
 
 class TabLabelEventBox(Gtk.EventBox):
-
     def __init__(self, notebook, text, settings):
         super().__init__()
         self.notebook = notebook
@@ -623,11 +636,11 @@ class TabLabelEventBox(Gtk.EventBox):
         self.label = Gtk.Label(text, visible=True)
         self.close_button = Gtk.Button(
             image=Gtk.Image.new_from_icon_name("window-close", Gtk.IconSize.MENU),
-            relief=Gtk.ReliefStyle.NONE
+            relief=Gtk.ReliefStyle.NONE,
         )
-        self.close_button.connect('clicked', self.on_close)
+        self.close_button.connect("clicked", self.on_close)
         settings.general.bind(
-            'tab-close-buttons', self.close_button, 'visible', Gio.SettingsBindFlags.GET
+            "tab-close-buttons", self.close_button, "visible", Gio.SettingsBindFlags.GET
         )
         self.box.pack_start(self.label, True, True, 0)
         self.box.pack_end(self.close_button, False, False, 0)
@@ -657,7 +670,7 @@ class TabLabelEventBox(Gtk.EventBox):
                 menu.popup(None, None, None, None, event.button, event.get_time())
             return True
         if event.button == 2:
-            prompt_cfg = self.notebook.guake.settings.general.get_int('prompt-on-close-tab')
+            prompt_cfg = self.notebook.guake.settings.general.get_int("prompt-on-close-tab")
             self.notebook.delete_page_by_label(self, prompt=prompt_cfg)
             return True
         if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
@@ -683,6 +696,18 @@ class TabLabelEventBox(Gtk.EventBox):
 
         self.grab_focus_on_last_focused_terminal()
 
+    @save_tabs_when_changed
+    def on_reset_custom_colors(self, user_data):
+        HidePrevention(self.get_toplevel()).prevent()
+        if PromptResetColorsDialog(self.notebook.guake.window).reset_tab_custom_colors():
+            page_num = self.notebook.find_tab_index_by_label(self)
+            for t in self.notebook.get_nth_page(page_num).iter_terminals():
+                t.reset_custom_colors()
+            self.notebook.guake.set_colors_from_settings_on_page(page_num=page_num)
+        HidePrevention(self.get_toplevel()).allow()
+
+        self.grab_focus_on_last_focused_terminal()
+
     def on_close(self, user_data):
-        prompt_cfg = self.notebook.guake.settings.general.get_int('prompt-on-close-tab')
+        prompt_cfg = self.notebook.guake.settings.general.get_int("prompt-on-close-tab")
         self.notebook.delete_page_by_label(self, prompt=prompt_cfg)
