@@ -19,7 +19,6 @@ Boston, MA 02110-1301 USA
 """
 
 from guake.about import AboutDialog
-from guake.boxes import DualTerminalBox
 from guake.boxes import RootTerminalBox
 from guake.boxes import TabLabelEventBox
 from guake.boxes import TerminalBox
@@ -33,14 +32,14 @@ from guake.utils import save_tabs_when_changed
 
 import gi
 import os
-gi.require_version('Gtk', '3.0')
-gi.require_version('Wnck', '3.0')
+
+gi.require_version("Gtk", "3.0")
+gi.require_version("Wnck", "3.0")
 from gi.repository import GObject
 from gi.repository import Gdk
 from gi.repository import Gtk
 from gi.repository import Wnck
 from guake.terminal import GuakeTerminal
-from locale import gettext as _
 
 import logging
 import posix
@@ -49,7 +48,6 @@ log = logging.getLogger(__name__)
 
 
 class TerminalNotebook(Gtk.Notebook):
-
     def __init__(self, *args, **kwargs):
         Gtk.Notebook.__init__(self, *args, **kwargs)
         self.last_terminal_focused = None
@@ -66,27 +64,48 @@ class TerminalNotebook(Gtk.Notebook):
         self.set_property("is-focus", True)
         self.set_property("expand", True)
 
-        if GObject.signal_lookup('terminal-spawned', TerminalNotebook) == 0:
+        if GObject.signal_lookup("terminal-spawned", TerminalNotebook) == 0:
             GObject.signal_new(
-                'terminal-spawned', TerminalNotebook, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE,
-                (GObject.TYPE_PYOBJECT, GObject.TYPE_INT)
+                "terminal-spawned",
+                TerminalNotebook,
+                GObject.SIGNAL_RUN_LAST,
+                GObject.TYPE_NONE,
+                (GObject.TYPE_PYOBJECT, GObject.TYPE_INT),
             )
             GObject.signal_new(
-                'page-deleted', TerminalNotebook, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ()
+                "page-deleted",
+                TerminalNotebook,
+                GObject.SIGNAL_RUN_LAST,
+                GObject.TYPE_NONE,
+                (),
             )
 
         self.scroll_callback = NotebookScrollCallback(self)
         self.add_events(Gdk.EventMask.SCROLL_MASK)
-        self.connect('scroll-event', self.scroll_callback.on_scroll)
+        self.connect("scroll-event", self.scroll_callback.on_scroll)
         self.notebook_on_button_press_id = self.connect(
             "button-press-event", self.on_button_press, None
         )
 
+        # Action box
         self.new_page_button = Gtk.Button(
-            image=Gtk.Image.new_from_icon_name("tab-new", Gtk.IconSize.MENU), visible=True
+            image=Gtk.Image.new_from_icon_name("tab-new-symbolic", Gtk.IconSize.MENU),
+            visible=True,
         )
         self.new_page_button.connect("clicked", self.on_new_tab)
-        self.set_action_widget(self.new_page_button, Gtk.PackType.END)
+
+        self.tab_selection_button = Gtk.Button(
+            image=Gtk.Image.new_from_icon_name("pan-down-symbolic", Gtk.IconSize.MENU),
+            visible=True,
+        )
+        self.popover = Gtk.Popover()
+        self.popover_window = None
+        self.tab_selection_button.connect("clicked", self.on_tab_selection)
+
+        self.action_box = Gtk.Box(visible=True)
+        self.action_box.pack_start(self.new_page_button, 0, 0, 0)
+        self.action_box.pack_end(self.tab_selection_button, 0, 0, 0)
+        self.set_action_widget(self.action_box, Gtk.PackType.END)
 
     def attach_guake(self, guake):
         self.guake = guake
@@ -101,12 +120,110 @@ class TerminalNotebook(Gtk.Notebook):
             except AttributeError:
                 # Gtk 3.18 fallback ("'Menu' object has no attribute 'popup_at_pointer'")
                 menu.popup(None, None, None, None, event.button, event.time)
-
-        elif event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS and event.button == 1:
+        elif (
+            event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS
+            and event.button == 1
+            and event.window.get_height() < 60
+        ):
+            # event.window.get_height() reports the height of the clicked frame
             self.new_page_with_focus()
 
         return False
 
+    @save_tabs_when_changed
+    def on_new_tab(self, user_data):
+        self.new_page_with_focus()
+
+    def on_tab_selection(self, user_data):
+        """Construct the tab selection popover
+
+        Since we did not use Gtk.ListStore to store tab information, we will construct the
+        tab selection popover content each time when user click them.
+        """
+
+        # Remove previous window
+        if self.popover_window:
+            self.popover.remove(self.popover_window)
+
+        # This makes the list's background transparent
+        # ref: epiphany
+        css_provider = Gtk.CssProvider()
+        css_provider.load_from_data(
+            b"#popover-window list { border-style: none; background-color: transparent; }"
+        )
+        Gtk.StyleContext.add_provider_for_screen(
+            Gdk.Screen.get_default(),
+            css_provider,
+            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION,
+        )
+
+        # Construct popover properties
+        BOX_HEIGHT = 30
+        LISTBOX_MARGIN = 12
+        self.popover_window = Gtk.ScrolledWindow(name="popover-window")
+        self.popover_listbox = Gtk.ListBox()
+        self.popover_listbox.set_property("margin", LISTBOX_MARGIN)
+        self.popover_window.add_with_viewport(self.popover_listbox)
+
+        max_height = (
+            self.guake.window.get_allocation().height - BOX_HEIGHT
+            if self.guake
+            else BOX_HEIGHT * 10
+        )
+        height = BOX_HEIGHT * self.get_n_pages() + LISTBOX_MARGIN * 4
+        self.popover_window.set_min_content_height(min(max_height, height))
+        self.popover_window.set_min_content_width(325)
+        self.popover.add(self.popover_window)
+
+        # Construct content
+        current_term = self.get_current_terminal()
+        selected_row = 0
+        for i in range(self.get_n_pages()):
+            row = Gtk.ListBoxRow()
+            box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+            box.set_size_request(200, BOX_HEIGHT)
+            label = Gtk.Label(self.get_tab_text_index(i))
+            label.set_xalign(0.0)
+            box.pack_start(label, 0, 0, 5)
+            row.add(box)
+            setattr(row, "page_index", i)
+            self.popover_listbox.add(row)
+            if current_term in self.get_terminals_for_page(i):
+                self.popover_listbox.select_row(row)
+                selected_row = i
+
+        # Signal
+        self.popover_listbox.connect("row-activated", self.on_popover_tab_select)
+
+        # Show popup
+        self.popover.set_position(Gtk.PositionType.TOP)
+        self.popover.set_relative_to(user_data)
+        self.popover.show_all()
+        try:
+            # For GTK >= 3.22
+            self.popover.popup()
+        except AttributeError:
+            pass
+
+        # Adjust scrollor
+        while Gtk.events_pending():
+            Gtk.main_iteration()
+
+        if selected_row:
+            adj = self.popover_window.get_vadjustment()
+            v = adj.get_upper() - adj.get_page_size()
+            part = v / self.get_n_pages()
+            adj.set_value(part * (selected_row + 1))
+
+    def on_popover_tab_select(self, list_box, row):
+        page_index = getattr(row, "page_index", -1)
+        if page_index != -1:
+            self.set_current_page(page_index)
+            self.get_terminals_for_page(page_index)[0].grab_focus()
+
+    def set_tabbar_visible(self, v):
+        self.set_property("show-tabs", v)
+
     def set_last_terminal_focused(self, terminal):
         self.last_terminal_focused = terminal
 
@@ -151,7 +268,8 @@ class TerminalNotebook(Gtk.Notebook):
                     total_procs += 1
             except OSError:
                 log.debug(
-                    "Cannot retrieve any pid from terminal %s, looks like it is already dead", index
+                    "Cannot retrieve any pid from terminal %s, looks like it is already dead",
+                    index,
                 )
                 return 0
         return total_procs
@@ -183,7 +301,7 @@ class TerminalNotebook(Gtk.Notebook):
         procs = self.get_running_fg_processes_count_page(page_num)
         if prompt == 2 or (prompt == 1 and procs > 0):
             # TODO NOTEBOOK remove call to guake
-            if not PromptQuitDialog(self.guake.window, procs, -1).close_tab():
+            if not PromptQuitDialog(self.guake.window, procs, -1, None).close_tab():
                 return
 
         page = self.get_nth_page(page_num)
@@ -212,7 +330,9 @@ class TerminalNotebook(Gtk.Notebook):
             page = self.get_nth_page(self.get_current_page())
             if page.get_terminals():
                 page.get_terminals()[0].grab_focus()
-        self.emit('page-deleted')
+
+        self.hide_tabbar_if_one_tab()
+        self.emit("page-deleted")
 
     def delete_page_by_label(self, label, kill=True, prompt=0):
         self.delete_page(self.find_tab_index_by_label(label), kill, prompt)
@@ -220,37 +340,66 @@ class TerminalNotebook(Gtk.Notebook):
     def delete_page_current(self, kill=True, prompt=0):
         self.delete_page(self.get_current_page(), kill, prompt)
 
-    def new_page(self, directory=None):
-        terminal = self.terminal_spawn(directory)
+    def new_page(self, directory=None, position=None, empty=False):
         terminal_box = TerminalBox()
-        terminal_box.set_terminal(terminal)
+        if empty:
+            terminal = None
+        else:
+            terminal = self.terminal_spawn(directory)
+            terminal_box.set_terminal(terminal)
         root_terminal_box = RootTerminalBox(self.guake, self)
         root_terminal_box.set_child(terminal_box)
-        page_num = self.append_page(root_terminal_box, None)
+        page_num = self.insert_page(
+            root_terminal_box, None, position if position is not None else -1
+        )
         self.set_tab_reorderable(root_terminal_box, True)
         self.show_all()  # needed to show newly added tabs and pages
+        # this is needed because self.window.show_all() results in showing every
+        # thing which includes the scrollbar too
+        self.guake.settings.general.triggerOnChangedValue(
+            self.guake.settings.general, "use-scrollbar"
+        )
         # this is needed to initially set the last_terminal_focused,
         # one could also call terminal.get_parent().on_terminal_focus()
-        self.terminal_attached(terminal)
+        if not empty:
+            self.terminal_attached(terminal)
+        self.hide_tabbar_if_one_tab()
+
+        if self.guake:
+            # Attack background image draw callback to root terminal box
+            root_terminal_box.connect_after("draw", self.guake.background_image_manager.draw)
         return root_terminal_box, page_num, terminal
 
+    def hide_tabbar_if_one_tab(self):
+        """Hide the tab bar if hide-tabs-if-one-tab is true and there is only one
+        notebook page"""
+        if self.guake.settings.general.get_boolean("window-tabbar"):
+            if self.guake.settings.general.get_boolean("hide-tabs-if-one-tab"):
+                self.set_property("show-tabs", self.get_n_pages() != 1)
+            else:
+                self.set_property("show-tabs", True)
+
     def terminal_spawn(self, directory=None):
         terminal = GuakeTerminal(self.guake)
         terminal.grab_focus()
+        terminal.connect(
+            "key-press-event",
+            lambda x, y: self.guake.accel_group.activate(x, y) if self.guake.accel_group else False,
+        )
         if not isinstance(directory, str):
-            directory = os.environ['HOME']
+            directory = os.environ["HOME"]
             try:
-                if self.guake.settings.general.get_boolean('open-tab-cwd'):
+                if self.guake.settings.general.get_boolean("open-tab-cwd"):
                     # Do last focused terminal still alive?
                     active_terminal = self.get_current_terminal()
                     if not active_terminal:
                         # If not alive, can we get any focused terminal?
                         active_terminal = self.get_focused_terminal()
-                    directory = os.path.expanduser('~')
+                    directory = os.path.expanduser("~")
                     if active_terminal:
                         # If found, we will use its directory as new terminal's directory
                         directory = active_terminal.get_current_directory()
-            except Exception as e:
+            except BaseException:
                 pass
         log.info("Spawning new terminal at %s", directory)
         terminal.spawn_sync_pid(directory)
@@ -258,16 +407,19 @@ class TerminalNotebook(Gtk.Notebook):
 
     def terminal_attached(self, terminal):
         terminal.emit("focus", Gtk.DirectionType.TAB_FORWARD)
-        self.emit('terminal-spawned', terminal, terminal.pid)
+        self.emit("terminal-spawned", terminal, terminal.pid)
 
-    def new_page_with_focus(self, directory=None, label=None, user_set=False):
-        box, page_num, terminal = self.new_page(directory)
+    def new_page_with_focus(
+        self, directory=None, label=None, user_set=False, position=None, empty=False
+    ):
+        box, page_num, terminal = self.new_page(directory, position=position, empty=empty)
         self.set_current_page(page_num)
         if not label:
             self.rename_page(page_num, _("Terminal"), False)
         else:
             self.rename_page(page_num, label, user_set)
-        terminal.grab_focus()
+        if terminal is not None:
+            terminal.grab_focus()
         return box, page_num, terminal
 
     def rename_page(self, page_index, new_text, user_set=False):
@@ -283,7 +435,7 @@ class TerminalNotebook(Gtk.Notebook):
             else:
                 label = TabLabelEventBox(self, new_text, self.guake.settings)
                 label.add_events(Gdk.EventMask.SCROLL_MASK)
-                label.connect('scroll-event', self.scroll_callback.on_scroll)
+                label.connect("scroll-event", self.scroll_callback.on_scroll)
 
                 self.set_tab_label(page, label)
             if user_set:
@@ -308,10 +460,6 @@ class TerminalNotebook(Gtk.Notebook):
     def get_tab_text_page(self, page):
         return self.get_tab_label(page).get_text()
 
-    @save_tabs_when_changed
-    def on_new_tab(self, user_data):
-        self.new_page_with_focus()
-
     def on_show_preferences(self, user_data):
         self.guake.hide()
         PrefsDialog(self.guake.settings).show()
@@ -335,12 +483,12 @@ class TerminalNotebook(Gtk.Notebook):
             flags=Gtk.DialogFlags.MODAL,
             buttons=Gtk.ButtonsType.OK_CANCEL,
             message_format=_(
-                'You are going to restore *all* the tabs!\n'
-                'which means all your terminals & pages '
-                'will be replaced.\n\nDo you want to continue?'
-            )
+                "You are going to restore *all* the tabs!\n"
+                "which means all your terminals & pages "
+                "will be replaced.\n\nDo you want to continue?"
+            ),
         )
-        dialog.connect('response', self.restore_tabs_dialog_response)
+        dialog.connect("response", self.restore_tabs_dialog_response)
         dialog.show()
 
     def restore_tabs_dialog_response(self, widget, response_id):
@@ -350,15 +498,22 @@ class TerminalNotebook(Gtk.Notebook):
 
 
 class NotebookManager(GObject.Object):
-
     def __init__(
-        self, window, notebook_parent, workspaces_enabled, terminal_spawned_cb, page_deleted_cb
+        self,
+        window,
+        notebook_parent,
+        workspaces_enabled,
+        terminal_spawned_cb,
+        page_deleted_cb,
     ):
         GObject.Object.__init__(self)
-        if not GObject.signal_lookup('notebook-created', self):
+        if not GObject.signal_lookup("notebook-created", self):
             GObject.signal_new(
-                'notebook-created', self, GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE,
-                (GObject.TYPE_PYOBJECT, GObject.TYPE_INT)
+                "notebook-created",
+                self,
+                GObject.SIGNAL_RUN_LAST,
+                GObject.TYPE_NONE,
+                (GObject.TYPE_PYOBJECT, GObject.TYPE_INT),
             )
         self.current_notebook = 0
         self.notebooks = {}
@@ -380,9 +535,9 @@ class NotebookManager(GObject.Object):
     def get_notebook(self, workspace_index: int):
         if not self.has_notebook_for_workspace(workspace_index):
             self.notebooks[workspace_index] = TerminalNotebook()
-            self.emit('notebook-created', self.notebooks[workspace_index], workspace_index)
-            self.notebooks[workspace_index].connect('terminal-spawned', self.terminal_spawned_cb)
-            self.notebooks[workspace_index].connect('page-deleted', self.page_deleted_cb)
+            self.emit("notebook-created", self.notebooks[workspace_index], workspace_index)
+            self.notebooks[workspace_index].connect("terminal-spawned", self.terminal_spawned_cb)
+            self.notebooks[workspace_index].connect("page-deleted", self.page_deleted_cb)
             log.info("created fresh notebook for workspace %d", self.current_notebook)
 
             # add a tab if there is none
@@ -395,7 +550,7 @@ class NotebookManager(GObject.Object):
         return self.get_notebook(self.current_notebook)
 
     def has_notebook_for_workspace(self, workspace_index):
-        return workspace_index in self.notebooks.keys()
+        return workspace_index in self.notebooks
 
     def set_workspace(self, index: int):
         self.notebook_parent.remove(self.get_current_notebook())
@@ -403,13 +558,19 @@ class NotebookManager(GObject.Object):
         log.info("current workspace is %d", self.current_notebook)
         notebook = self.get_current_notebook()
         self.notebook_parent.add(notebook)
-        if self.window.get_property('visible') and \
-                notebook.last_terminal_focused is not None:
+        if self.window.get_property("visible") and notebook.last_terminal_focused is not None:
             notebook.last_terminal_focused.grab_focus()
 
         # Restore pending page terminal split
         notebook.guake.restore_pending_terminal_split()
 
+        # Restore config to workspace
+        notebook.guake.load_config()
+
+    def set_notebooks_tabbar_visible(self, v):
+        for nb in self.iter_notebooks():
+            nb.set_tabbar_visible(v)
+
     def get_notebooks(self):
         return self.notebooks
 
@@ -424,6 +585,12 @@ class NotebookManager(GObject.Object):
             for t in self.notebooks[k].iter_terminals():
                 yield t
 
+    def get_terminal_by_uuid(self, terminal_uuid):
+        for t in self.iter_terminals():
+            if t.uuid == terminal_uuid:
+                return t
+        return None
+
     def iter_pages(self):
         for k in self.notebooks:
             for t in self.notebooks[k].iter_pages():
